# sb::Plugin::PerCategory - Serene Bach Plugin
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

# 0.00 [2007/06/08] generated
# 0.01 [2007/06/09] declared resource
# 0.02 [2007/07/12] added management screen

package sb::Plugin::PerCategory;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'cms',
		'name'    => 'Entries Per Category',
		'text'    => 'Displaying entries for each category.',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => '0.02',
	},
	'file' => 'per_category.txt',
	'data' => 1,
);
sb::Plugin->register_admin_module(
	'mode'   => 'percat',
	'level'  => 1,
	'module' => 'sb::Admin::PerCategory',
);
sb::Plugin->register_content_module(
	'type'     => 'main',
	'callback' => \&sb::Content::PerCategory::_main,
	'field'    => 'per_category',
);
package sb::Content::PerCategory;
use sb::Time ();
sub _main
{
	my $cms = shift;
	my %var = @_;
	return( 0 ) if ($var{'mode'} ne 'page'); # available on page mode.
	my $count = _tree_per_category(
		'cat'    => $var{'sortedcat'},
		'branch' => undef,
		'cms'    => $cms,
		'var'    => \%var,
	);
	if ($count > 0)
	{
		$cms->block('per_category_available'=>1);
	}
	else
	{
		$cms->block('per_category_not_available'=>1);
	}
	return( $count );
}
sub _tree_per_category
{
	my %param = (
		'cat'    => [],
		'branch' => undef,
		'cms'    => undef,
		'var'    => undef,
		'count'  => 0,
		@_
	);
	return( 0 ) if (!$param{'cms'} or !$param{'var'});
	my $data = sb::Plugin->get_data;
	my @sets = split(',',$data->data);
	my $cms = $param{'cms'};
	my %var = %{$param{'var'}};
	CATEGORY_LOOP:
	foreach my $cat ( @{$param{'cat'}} )
	{
		my $id = $cat->id;
		next if (!defined($param{'branch'}) and $cat->main() ne '');
		next if ( defined($param{'branch'}) and $cat->main() ne $param{'branch'});
		my $flag = $data->setting;
		if ($flag)
		{
			$flag = grep(/^\Q$id\E$/,@sets);
		}
		else
		{
			$flag = ($cat->get_option('top') != 1 and $cat->get_option('list') != 1);
		}
		if ($flag)
		{
			my @ents = sb::Data->load('Entry',
				'cond'  => {'stat'=>1,'cat'=>$id},
				'sort'  => 'date',
				'order' => 1,
				'num'   => $var{'conf'}->value('conf_newent_disp'),
			);
			if (@ents)
			{ # we have entries, so we should handle them
				$cms->num($param{'count'});
				$cms->tag('percat_category_fullname'=>$cat->fullname($var{'cat'}));
				$cms->tag('percat_category_link'=>$cat->cat_url());
				$cms->tag('percat_category_name'=>$cat->name());
				$cms->tag('percat_category_description'=>$cat->formated_text());
				$cms->tag('percat_category_id'=>$id);
				$cms->tag('entry_list_percat'=>_entrylist(\%var,\@ents));
				$param{'count'}++;
			}
		}
		if ($cat->sub ne '')
		{
			$param{'count'} = _tree_per_category(
				'cat'    => $param{'cat'},
				'branch' => $id,
				'var'    => $param{'var'},
				'cms'    => $param{'cms'},
				'count'  => $param{'count'},
			);
		}
	} # end of CATEGORY_LOOP
	return( $param{'count'} );
}
sub _entrylist
{
	my %var  = %{shift()};
	my @ents = @{shift()};
	my $list = '<ul>' . "\n";
	foreach my $ent ( @ents )
	{
		my $url = $ent->permalink('cat'=>$var{'cat'},'type'=>$var{'conf'}->value('conf_entry_archive'));
		my $date = sb::Time->format(
			'time'=>$ent->date,
			'form'=>$var{'conf'}->value('conf_dateinlist'),
			'zone'=>$var{'conf'}->value('conf_timezone')
		);
		$list .= '<li><a href="' . $url . '">' . $ent->subj() . '</a>' . $date . '</li>' . "\n";
	}
	$list .= '</ul>' . "\n";
	return( $list );
}
package sb::Admin::PerCategory;
# --------------------------------------------------
# // declaration for global variables
use vars qw( @ISA );
# ---------------------------------------------------
# // configuration for inheritance / dependancy
use sb::Language ();
use sb::Interface ();
use sb::TemplateManager ();
use sb::Text ();
use sb::Admin::Entry ();
@ISA = qw( sb::Admin::Entry );
# --------------------------------------------------
# // declaration for constant value
sub TEMPLATE  (){ 'per_category.html' };
sub LIST_TEMP (){ '<input type="checkbox" name="sel" value="%d"%s onclick="switchList(this)" />' }
sub PRETXT    (){ '&nbsp;&gt;&nbsp;' };
# ---------------------------------------------------
# // public functions - callback
sub callback
{
	my $self = shift;
	return ( $self->{'regi'} ) 
		? $self->_save_setting(@_)
		: $self->_open_setting(@_);
}
# ---------------------------------------------------
# // private functions - main routine
sub _save_setting
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi  = sb::Interface->get;
	my $data = sb::Plugin->get_data;
	my @selected = split("\0",$cgi->value('sel'));
	$data->data(join(',',@selected));
	$data->setting(($cgi->value('use_this') ne '') ? 1 : 0);
	sb::Plugin->set_data('data'=>$data);
	return $self->_open_setting('message'=>sb::Language->get->string('parts_editcomp'));
}
sub _open_setting
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi  = sb::Interface->get;
	my $cms  = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $data = sb::Plugin->get_data;
	$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	$cms->num(0);
	$cms->tag('sb_use_this'=>($data->setting) ? 'checked="checked"' : '');
	$cms->block('sb_category_list' => $self->_category_tree(
			'cms' => $cms,
			'cat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
			'sel' => [split(',',$data->data)],
		)
	);
	$self->common_template_parts($cms);
	if ($param{'message'} ne '')
	{
		$cms->num(0);
		$cms->tag('sb_process_message'=>$param{'message'});
		$cms->block('sb_process_message'=>1);
	}
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}
sub _category_tree
{
	my $self  = shift;
	my %param = (
		'cms'    => undef,
		'cat'    => [],
		'sel'    => [],
		'branch' => undef,
		'level'  => 0,
		'class'  => -1,
		@_
	);
	return if (!$param{'cms'});
	my $num = $param{'class'};
	foreach my $cat ( @{$param{'cat'}} )
	{
		next if (!defined($param{'branch'}) and $cat->main ne '');
		next if ( defined($param{'branch'}) and $cat->main ne $param{'branch'});
		$num++;
		my $id = $cat->id;
		my $checked = ' checked="checked"' if ( grep(/^\Q$id\E$/,@{$param{'sel'}}) );
		my $class = ($num % 2) ? 'odd' : 'even';
		$class = 'selected_' . $class if ($checked);
		$param{'cms'}->num($num);
		$param{'cms'}->tag('sb_list_class'   => $class);
		$param{'cms'}->tag('sb_catlist_id'   => $id);
		$param{'cms'}->tag('sb_catlist_name' => (PRETXT x $param{'level'}) . $cat->name);
		$param{'cms'}->tag('sb_catlist_sel'  => sprintf(LIST_TEMP,$cat->id,$checked));
		if ($cat->sub ne '')
		{
			$num = $self->_category_tree(
				'cms'    => $param{'cms'},
				'cat'    => $param{'cat'},
				'sel'    => $param{'sel'},
				'branch' => $id,
				'level'  => $param{'level'} + 1,
				'class'  => $num,
			);
		}
	}
	return($num);
}
1;
__END__
